;  ____   __    ___     __  __    __    _  _ 
; (  _ \ /__\  / __)___(  \/  )  /__\  ( \( )
;  )___//(__)\( (__(___))    (  /(__)\  )  ( 
; (__) (__)(__)\___)   (_/\/\_)(__)(__)(_)\_)
;
; Channel F Pac-Man Port written by Blackbird, 2004-2006
; Pac-Man Copyright (c) 1980 Namco Ltd.

	processor f8

;[TODO] make directions stored as two, not four bits
;[TODO] ghost looping function to space time/space? (spacetime?)
;[TODO] ghost drawing/clearing function?
;[TODO] erase ghosts first, then redraw them
; add in dying routine
; enable ghost slowdown
; add in blit bitmasking / tunnel masking
; add in extended ghost logic
; enable power pellets

;===========================================================================
; VES Header
;===========================================================================

	include	"ves.h"	

;===========================================================================
; Configuration
;===========================================================================

game_size		=	4				; game size in kilobytes

switch_music		=	1				; switches music off or on
switch_sfx		=	0				; switches sfx off or on

;---------------------------------------------------------------------------
; Register Reference 
;---------------------------------------------------------------------------

; The registers in the F8 are used as both RAM and registers for
; this game. The reference is as follows:

; score must have a 4 in the octal one's digit, to work with score.add
HISCORE_REG		=	020
SCORE_REG		=	024

; pacman registers (must be in same octet)
PACMAN_X_REG		=	030
PACMAN_Y_REG		=	031
PACMAN_DIRECTION_REG	=	032
PACMAN_ANIMINDEX_REG	=	033

; ghost registers (must be in same octet)
GHOST_TEMP_REG		=	034
GHOST_X_REG		=	034
GHOST_Y_REG		=	035
GHOST_DIRECTION_REG	=	036
GHOST_ANIMINDEX_REG	=	037

; ghost registers (040-057)
GHOST_REG		=	040

; level registers (must be in same octet)
LEVEL_NUMBER_REG	=	060
GHOST_NUMBER_REG	=	061
PACMAN_LIVES_REG	=	062
GHOSTS_SLOW_REG		=	063				; upper 4 bits
POWERPELLETS_REG	=	063				; lower 4 bits

;---------------------------------------------------------------------------
; Variables 
;---------------------------------------------------------------------------

; Pac-Man
PACMAN_COLOR		=	green
PACMAN_START_X		=	39
PACMAN_START_Y		=	41
PACMAN_LIVES		=	4

; distance from ghost required for death
DEATH_DISTANCE		=	2

; maze
MAZE_BGCOLOR		=	bkg
MAZE_FGCOLOR		=	blue
PELLET_COLOR		=	red
POWERPELLET_COLOR	=	red
GATE_COLOR		=	red

; sidebar
SIDEBAR_COLOR		=	blue
SCORE_COLOR		=	green

; ghosts
GHOST_NUMBER		=	4
GHOST_1_COLOR		=	red
GHOST_2_COLOR		=	green
GHOST_3_COLOR		=	blue
GHOST_4_COLOR		=	red

;===========================================================================
; Program Entry
;===========================================================================

;---------------------------------------------------------------------------
; Cartridge Initalization
;---------------------------------------------------------------------------

	org	$800

cartridge.init:
	; initalize the system
	CARTRIDGE_START
	CARTRIDGE_INIT
	
	; initialize the kstack pointer
	li	62		; stack starts at r62
	lisu	7		; stack pointer is r63
	lisl	7
	lr	S, A

;---------------------------------------------------------------------------
; Main Program 
;---------------------------------------------------------------------------

main:
	; display a three color, grey background
	li	$c6
	lr	3, A
	pi	clrscrn
				
main.startLevel:
	; load in the level number
	SETISAR	LEVEL_NUMBER_REG
	lis	1						; we're on level 0
	lr	S, A

	; clear the hiscore
	SETISAR	HISCORE_REG
	clr
	lr	I, A
	lr	I, A
	lr	I, A
	lr	I, A
	; clear the score
	lr	I, A
	lr	I, A
	lr	I, A
	lr	S, A

	; load pacman's number of lives
	SETISAR	PACMAN_LIVES_REG
	lis	PACMAN_LIVES
	lr	S, A	

;===========================================================================
; level loop
;===========================================================================

;---------------------------------------------------------------------------
; init 
;---------------------------------------------------------------------------

level.init:
	; setup the pellet RAM
	pi	pellets.init
	pi	powerPellets.init

level.init.drawScreen:
	; draw the map
	pi	map.draw
	; sidebar
	pi	sidebar.draw
	; pellets
	pi	pellets.draw
	; power pellets
	pi	powerPellets.draw
	; player scores
	pi	score.draw
	pi	hiscore.draw

	; draw "PLAYER 1"
	dci	gfx.player1.parameters
	pi	blitGraphic

level.init.intro:
	; draw "READY!"
	dci	gfx.ready.parameters
	pi	blitGraphic

	; draw the number of lives
	pi	pacman.drawLives

	; check that we're starting a new game (the score would be 0)
	SETISAR	SCORE_REG
	clr
	xs	I
	bnz	level.init.setupPacman
	xs	I
	bnz	level.init.setupPacman
	xs	I
	bnz	level.init.setupPacman
	xs	S
	bnz	level.init.setupPacman

    IF switch_music
level.init.intro.playSong.0:
	; play the opening song
	dci	sfx.intro.0
	pi	playSong
    ENDIF

	; subtract pacman's current life
	SETISAR	PACMAN_LIVES_REG
	ds	I
	; draw them on-screen
	pi	pacman.drawLives
   
level.init.setupPacman:
	; load coordinates
	SETISAR	PACMAN_X_REG
	li	PACMAN_START_X
	lr	I, A						; load the pac-man x coordinate
	li	PACMAN_START_Y
	lr	S, A						; load the pac-man y coordinate

	; set the direction
	SETISAR	PACMAN_DIRECTION_REG
	lis	%0001
	lr	S, A

	; load the animation index
	SETISAR	PACMAN_ANIMINDEX_REG
	lis	2
	lr	S, A

level.init.setupGhosts:
	; setup the ghosts
	SETISAR	GHOST_REG
	
level.init.setupGhosts.1:
	; x position
	li	39
	lr	I, A
	; y position
	li	17
	lr	I, A
	; direction
	lis	%0001
	lr	I, A
	; animation index
	lis	0
	lr	I, A

level.init.setupGhosts.2:
	; x position
	li	30
	lr	I, A
	; y position
	li	23 + 6
	lr	I, A
	; direction
	lis	%0001
	lr	I, A
	; animation index
	lis	0
	lr	I, A

	SETISAR	[GHOST_REG + 010]
level.init.setupGhosts.3:
	; x position
	li	39
	lr	I, A
	; y position
	li	23 + 6
	lr	I, A
	; direction
	lis	%0001
	lr	I, A
	; animation index
	lis	0
	lr	I, A

level.init.setupGhosts.4:
	; x position
	li	48
	lr	I, A
	; y position
	li	23 + 6
	lr	I, A
	; direction
	lis	%0010
	lr	I, A
	; animation index
	lis	0
	lr	I, A

	; clear player-text and ready text
level.init.text.erase:
 	; load color
	li	bkg
	lr	1, A
	; start row loop
	li	33
	lr	3, A
level.init.text.erase.row:
	li	51
	lr	2, A
level.init.text.erase.column:
	pi	plot

level.init.text.erase.checkLoop:
	; check column
	ds	2
	lr	A, 2
	ci	28						; see if we've reached the left edge
	bnz	level.init.text.erase.column
	; check row
	ds	3
	lr	A, 3
	ci	28
	bnz	level.init.text.erase.row			; branch if > 0

level.init.text2.erase:
 	; load color
	li	bkg
	lr	1, A
	; start row loop
	li	21
	lr	3, A
level.init.text2.erase.row:
	li	56
	lr	2, A
level.init.text2.erase.column:
	pi	plot

level.init.text2.erase.checkLoop:
	; check column
	ds	2
	lr	A, 2
	ci	26				; see if we've reached the left edge
	bnz	level.init.text2.erase.column
	; check row
	ds	3
	lr	A, 3
	ci	16
	bnz	level.init.text2.erase.row			; branch if > 0

level.init.drawPacman:
	; draw Pac-Man
	pi	pacman.draw

	; load first ghost
	SETISAR GHOST_NUMBER_REG
	lis	0
	lr	S, A
level.init.drawGhosts:
	; load this ghost's coordinates
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	sl	1
	sl	1
	ai	GHOST_REG
	lr	0, A

	; load the ghost color
	dci	gfx.ghost.colors
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	adc
	lm
	lr	1, A
	
	; load the saved ISAR
	lr	A, 0
	lr	IS ,A
	; load x coordinate
	lr	A, I
	lr	2, A
	; load y coordinate
	lr	A, S
	lr	3, A
	; set sprite index
	lr	A, I						; increase isar
	lr	A, S
	ni	%1
	ai	17
	lr	4, A
	; set bitmask
	li	%11111000
	lr	5, A
	; draw the sprite
	pi	sprite.draw

level.init.drawGhosts.checkLoop:
	; check if we've reached the last ghost
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	inc
	ci	GHOST_NUMBER
	bz	level.init.drawGhosts.end
	; update next ghost
	lr	S, A
	br	level.init.drawGhosts
level.init.drawGhosts.end:

    IF switch_music
	; check that we're starting a new game (the score would be 0)
	SETISAR	SCORE_REG
	clr
	xs	I
	bnz	level.init.delay
	xs	I
	bnz	level.init.delay
	xs	I
	bnz	level.init.delay
	xs	S
	bnz	level.init.delay

level.init.intro.playSong.1:
	; play the opening song
	dci	sfx.intro.1
	pi	playSong
	br	level.loop

level.init.delay:
	; delay for a short bit
	li	$ff
	lr	5, A
	pi	delay
	li	$ff
	lr	5, A
	pi	delay

    ENDIF
    
;---------------------------------------------------------------------------
; loop
;---------------------------------------------------------------------------

level.loop:
	; draw the player score
	pi	score.draw

;---------------------------------------------------------------------------
; pacman
;---------------------------------------------------------------------------

	; spool for controller input
level.loop.pacman.readController:
	; see if one of the hand controllers has moved
	clr
	outs	0
	outs	4
	ins	4
	com							; un-invert port data
	ni	%11001111					; mask off twists, since they're unused
	bnz	level.loop.pacman.readController.end
	; check the other controller (?)
	outs	1
	ins	1
	com
	ni	%11001111
level.loop.pacman.readController.end:
	; save the input
	lr	6, A

level.loop.pacman.handleInput:
	; check to see if we're on an intersection
	pi	pacman.checkIntersection
	; branch if false
	ci	1
	bnz	level.loop.pacman.handleInput.noIntersection

level.loop.pacman.handleInput.inIntersection:
	; ensure only one direction is selected
	lr	A, 6
	ni	%0111
	bz	level.loop.pacman.handleInput.inIntersection.end
	lr	6, A
	ni	%0011
	bz	level.loop.pacman.handleInput.inIntersection.end
	lr	6, A
	ni	%0001
	bz	level.loop.pacman.handleInput.inIntersection.end
	lr	6, A
level.loop.pacman.handleInput.inIntersection.end:
	; load the boundaries
	lr	A, 0
	; compare and set direction
	br	level.loop.pacman.handleInput.set

level.loop.pacman.handleInput.noIntersection:
	; if you're not in an intersection,
	; you can only go the opposite way
	; you're going right now

	; set ISAR for direction
	SETISAR	PACMAN_DIRECTION_REG
	; up
	lis	%1000
	ns	S
	bnz	level.loop.pacman.handleInput.up
	; down
	lis	%0100
	ns	S
	bnz	level.loop.pacman.handleInput.down
	; left
	lis	%0010
	ns	S
	bnz	level.loop.pacman.handleInput.left
	; right
	lis	%0001
	ns	S
	bnz	level.loop.pacman.handleInput.right
	; none
	clr
	br	level.loop.pacman.handleInput.set

level.loop.pacman.handleInput.up:
	lis	%0100
	br	level.loop.pacman.handleInput.set
level.loop.pacman.handleInput.down:
	lis	%1000
	br	level.loop.pacman.handleInput.set
level.loop.pacman.handleInput.left:
	lis	%0001
	br	level.loop.pacman.handleInput.set
level.loop.pacman.handleInput.right:
	lis	%0010
level.loop.pacman.handleInput.set:
	; see if the direction is allowed, otherwise end routine
	ns	6
	bz	level.loop.pacman.handleInput.end

	; set the direction register
	SETISAR	PACMAN_DIRECTION_REG
	lr	S, A

level.loop.pacman.handleInput.end:

level.loop.pacman.checkDirection:
	; check that pacman's current direction is still valid
	pi	pacman.checkIntersection
	ci	1
	bnz	level.loop.pacman.checkDirection.end			; not in an intersection

	; clear direction if necessary
	SETISAR	PACMAN_DIRECTION_REG
	lr	A, S
	ns	0
	bnz	level.loop.pacman.checkDirection.end
	br	level.loop.pacman.draw				; can't go anywhere, so just skip updating pacman	
level.loop.pacman.checkDirection.end:

	; clear pac-man
	pi	pacman.clear

level.loop.pacman.update:
	SETISAR	PACMAN_ANIMINDEX_REG
	lr	A, S						; load the sprite index
	inc							; increase it
	ni	%00000011					; clear if on frame 4
	lr	S, A						; save the new index

level.loop.pacman.move:
	; now we move pac-man based on the direction
	; which is set in r37

	SETISAR	PACMAN_DIRECTION_REG
	lr	A,S
	; right (%0001)
	sr	1
	bz	level.loop.pacman.move.right
	; left (%0010)
	sr	1
	bz	level.loop.pacman.move.left
	; down (%0100)
	sr	1
	bz	level.loop.pacman.move.down
	; up (%1000)
	sr	1
	bz	level.loop.pacman.move.up
	; none (%0000)
	br	level.loop.pacman.move.end

level.loop.pacman.move.up:
	; subtract y by 1
	SETISAR	PACMAN_Y_REG
	ds	S						; subtract 1
	br	level.loop.pacman.move.end
level.loop.pacman.move.right:
	; add x by 1
	SETISAR	PACMAN_X_REG
	lr	A, S
	inc							; add 1
	lr	S, A
	br	level.loop.pacman.move.end
level.loop.pacman.move.down:
	; add y by 1
	SETISAR	PACMAN_Y_REG
	lr	A, S
	inc							; add 1
	lr	S, A
	br	level.loop.pacman.move.end
level.loop.pacman.move.left:
	; subtract x by 1
	SETISAR	PACMAN_X_REG
	ds	S						; subtract 1
	br	level.loop.pacman.move.end
level.loop.pacman.move.end:

	; check if Pac-Man has moved through the tunnel
level.loop.pacman.checkTunnel:
	; check Y position
	SETISAR PACMAN_Y_REG
	lr	A, D
	ci	$17
	bnz	level.loop.pacman.checkTunnel.end
	; check X position
	lr	A, S
	ci	$48
	bz	level.loop.pacman.checkTunnel.warpRight		; Pac-Man has passed through left side
	ci	$6
	bnz	level.loop.pacman.checkTunnel.end

level.loop.pacman.checkTunnel.warpLeft:
	; set new position on right side
	li	$47
	lr	S, A
	br	level.loop.pacman.checkTunnel.end	
level.loop.pacman.checkTunnel.warpRight:
	; set new position on left side
	li	$7
	lr	S, A
level.loop.pacman.checkTunnel.end:

level.loop.pacman.draw:
	; draw Pac-Man
	pi	pacman.draw
	
;---------------------------------------------------------------------------
; ghosts 
;---------------------------------------------------------------------------

	; ghost loop; loops through each of the four ghosts updating
	; position and direction
	SETISAR GHOST_NUMBER_REG
	lis	0
	lr	S, A
level.loop.ghosts:
	
	; move registers to temp location
	; get ISAR location for ghost
	SETISAR GHOST_NUMBER_REG
	; multiply by 4 bytes per ghost
	lr	A, S
	sl	1
	sl	1
	ai	GHOST_REG
	lr	IS, A
	; load registers
	lr	A, I
	lr	0, A
	lr	A, I
	lr	1, A
	lr	A, I
	lr	2, A
	lr	A, S
	lr	3, A
	; store at temp ghost position
	SETISAR	GHOST_TEMP_REG
	lr	A, 0
	lr	I, A
	lr	A, 1
	lr	I, A
	lr	A, 2
	lr	I, A
	lr	A, 3
	lr	S, A

level.loop.ghosts.checkDirection:
	; check that the ghosts's current direction is still valid
	SETISAR	GHOST_X_REG
	lr	A, I
	lr	4, A
	lr	A, I
	lr	5, A
	pi	checkIntersection
	ci	1
	bz	level.loop.ghosts.checkDirection.inIntersection
	jmp	level.loop.ghosts.checkDirection.end			; not in an intersection

level.loop.ghosts.checkDirection.inIntersection:	
	; change direction; mask off reverse direction
	SETISAR	GHOST_DIRECTION_REG
	lr	A, S
	ni	%0111
	bz	level.loop.ghosts.checkDirection.mask.up
	ni	%0011
	bz	level.loop.ghosts.checkDirection.mask.down
	ni	%0001
	bz	level.loop.ghosts.checkDirection.mask.left
	; direction masking
level.loop.ghosts.checkDirection.mask.right:
	lis	%1101
	br	level.loop.ghosts.checkDirection.mask.end
level.loop.ghosts.checkDirection.mask.left:
	lis	%1110
	br	level.loop.ghosts.checkDirection.mask.end
level.loop.ghosts.checkDirection.mask.down:
	lis	%0111
	br	level.loop.ghosts.checkDirection.mask.end
level.loop.ghosts.checkDirection.mask.up:
	lis	%1011
level.loop.ghosts.checkDirection.mask.end:
	; save the masked direction
	ns	0
	lr	0, A
	
	; find the X distance, dX (r1)
	SETISAR	GHOST_X_REG
	lr	A, S
	com
	inc
	SETISAR	PACMAN_X_REG
	as	S
	lr	1, A
	; find shortX (r3) & longX (r4)
	ni	%10000000						; check if negative
	bnz	level.loop.ghosts.checkDirection.dX.negative
level.loop.ghosts.checkDirection.dX.positive:
	; shortX = right, longX = left
	lis	%0001
	lr	3, A
	lis	%0010
	lr	4, A
	br	level.loop.ghosts.checkDirection.dX.end
level.loop.ghosts.checkDirection.dX.negative:
	; shortX = left, longX = right
	lis	%0010
	lr	3, A
	lis	%0001
	lr	4, A
	; also get the absolute value of dX
	lr	A, 1
	com
	inc
	lr	1, A
level.loop.ghosts.checkDirection.dX.end:

	; find the Y distance, dY (r2)
	SETISAR	GHOST_Y_REG
	lr	A, S
	com
	inc
	SETISAR	PACMAN_Y_REG
	as	S
	lr	2, A
	; find shortY (r5) & longY (r6)
	ni	%10000000						; check if negative
	bnz	level.loop.ghosts.checkDirection.dY.negative
level.loop.ghosts.checkDirection.dY.positive:
	; shortY = down, longY = up
	lis	%0100
	lr	5, A
	lis	%1000
	lr	6, A
	br	level.loop.ghosts.checkDirection.dY.end
level.loop.ghosts.checkDirection.dY.negative:
	; shortY = up, longY = down
	lis	%1000
	lr	5, A
	lis	%0100
	lr	6, A
	; also get the absolute value of dY
	lr	A, 2
	com
	inc
	lr	2, A
level.loop.ghosts.checkDirection.dY.end:

	; check if the X distance is greater than the Y distance
	lr	A, 1
	com
	inc
	as	2
	bm	level.loop.ghosts.checkDirection.compare.gt
	; dX <= dY
level.loop.ghosts.checkDirection.compare.lt:
	; shortX
	lr	A, 3
	ns	0
	bnz	level.loop.ghosts.checkDirection.compare.end
	; shortY
	lr	A, 5
	ns	0
	bnz	level.loop.ghosts.checkDirection.compare.end
	; longX
	lr	A, 4
	ns	0
	bnz	level.loop.ghosts.checkDirection.compare.end
	; longY
	lr	A, 6
	ns	0
	bnz	level.loop.ghosts.checkDirection.compare.end
	
	; panic! this point should never be reached
	br	level.loop.ghosts.checkDirection.end
	
	; dX >= dY
level.loop.ghosts.checkDirection.compare.gt:
	; shortY
	lr	A, 5
	ns	0
	bnz	level.loop.ghosts.checkDirection.compare.end
	; shortX
	lr	A, 3
	ns	0
	bnz	level.loop.ghosts.checkDirection.compare.end
	; longY
	lr	A, 6
	ns	0
	bnz	level.loop.ghosts.checkDirection.compare.end
	; longX
	lr	A, 4
	ns	0
	bnz	level.loop.ghosts.checkDirection.compare.end
	
	; panic! this point should never be reached
	br	level.loop.ghosts.checkDirection.end

level.loop.ghosts.checkDirection.compare.end:
	; save the ghost's direction
	SETISAR	GHOST_DIRECTION_REG
	lr	S, A
	
level.loop.ghosts.checkDirection.end:

level.loop.ghosts.clear:
	; clear the ghost
	li	MAZE_BGCOLOR
	lr	1, A
	; load x coordinate
	SETISAR	GHOST_X_REG
	lr	A, I
	lr	2, A
	; load y coordinate
	lr	A, S
	lr	3, A
	; set sprite index
	li	0
	lr	4, A
	; set bitmask
	li	%11111000
	lr	5, A
	; draw the sprite
	pi	sprite.draw

level.loop.ghosts.redrawPellets:
	; redraw overlapped pellets
	SETISAR	GHOST_X_REG
	lr	A, I
	lr	1, A
	lr	A, D
	lr	2, A
	pi	ghost.checkPelletCollision
	; redraw overlapped powerpellets [TODO] fix this
	SETISAR	GHOST_X_REG
	lr	A, I
	lr	1, A
	lr	A, D
	lr	2, A
	pi	powerPellets.checkCollision

level.loop.ghosts.move:
	; move the ghosts using the direction set in r40

	SETISAR	GHOST_DIRECTION_REG
	lr	A, S
	; right (%0001)
	sr	1
	bz	level.loop.ghosts.move.right
	; left (%0010)
	sr	1
	bz	level.loop.ghosts.move.left
	; down (%0100)
	sr	1
	bz	level.loop.ghosts.move.down
	; up (%1000)
	sr	1
	bz	level.loop.ghosts.move.up
	; none (%0000)
	br	level.loop.ghosts.move.end

level.loop.ghosts.move.up:
	; subtract y by 1
	SETISAR	GHOST_Y_REG
	ds	S						; subtract 1
	br	level.loop.ghosts.move.end
level.loop.ghosts.move.right:
	; add x by 1
	SETISAR	GHOST_X_REG
	lr	A, S
	inc							; add 1
	lr	S, A
	br	level.loop.ghosts.move.end
level.loop.ghosts.move.down:
	; add y by 1
	SETISAR	GHOST_Y_REG
	lr	A, S
	inc							; add 1
	lr	S, A
	br	level.loop.ghosts.move.end
level.loop.ghosts.move.left:
	; subtract x by 1
	SETISAR	GHOST_X_REG
	ds	S						; subtract 1
	br	level.loop.ghosts.move.end
level.loop.ghosts.move.end:

	; check if the ghost has moved through the tunnel
level.loop.ghosts.move.checkTunnel:
	; check Y position
	SETISAR GHOST_Y_REG
	lr	A, D
	ci	$17
	bnz	level.loop.ghosts.move.checkTunnel.end
	; check X position
	lr	A, S
	ci	$48
	bz	level.loop.ghosts.move.checkTunnel.warpRight	; the ghost has passed through left side
	ci	$6
	bnz	level.loop.ghosts.move.checkTunnel.end

level.loop.ghosts.move.checkTunnel.warpLeft:
	; set new position on right side
	li	$47
	lr	S, A
	br	level.loop.ghosts.move.checkTunnel.end	
level.loop.ghosts.move.checkTunnel.warpRight:
	; set new position on left side
	li	$7
	lr	S, A
level.loop.ghosts.move.checkTunnel.end:
	
	; draw a ghost
level.loop.ghosts.draw:
	; load the ghost color
	dci	gfx.ghost.colors
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	adc
	lm
	lr	1, A
	; load x coordinate
	SETISAR	GHOST_X_REG
	lr	A, I
	lr	2, A
	; load y coordinate
	lr	A, S
	lr	3, A
	; set sprite index
	SETISAR	GHOST_ANIMINDEX_REG
	lr	A, S
	ni	%1
	ai	17
	lr	4, A
	; set bitmask
	li	%11111000
	lr	5, A
	; draw the sprite
	pi	sprite.draw
level.loop.ghosts.draw.end:

	; increase the animation index
	SETISAR	GHOST_ANIMINDEX_REG
	ds	S

	; restore modified registers
	SETISAR	GHOST_TEMP_REG
	; load registers
	lr	A, I
	lr	0, A
	lr	A, I
	lr	1, A
	lr	A, I
	lr	2, A
	lr	A, S
	lr	3, A
	; get ISAR location for ghost
	SETISAR GHOST_NUMBER_REG
	; multiply by 4 bytes per ghost
	lr	A, S
	sl	1
	sl	1
	ai	GHOST_REG
	lr	IS, A
	; store at temp ghost position
	lr	A, 0
	lr	I, A
	lr	A, 1
	lr	I, A
	lr	A, 2
	lr	I, A
	lr	A, 3
	lr	S, A

level.loop.ghosts.checkLoop:
	; check if we've reached the last ghost
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	inc
	ci	GHOST_NUMBER
	bz	level.loop.ghosts.end
	; update next ghost
	lr	S, A
	jmp	level.loop.ghosts

level.loop.ghosts.end:

;---------------------------------------------------------------------------
; pacman death
;---------------------------------------------------------------------------
; check if pacman has collided with a ghost

	; load first ghost
	SETISAR GHOST_NUMBER_REG
	lis	0
	lr	S, A
level.loop.checkPacmanCollision:
	; load this ghost's coordinates
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	sl	1
	sl	1
	ai	GHOST_REG
	lr	IS, A
	; x position
	lr	A, I
	lr	0, A
	; y position
	lr	A, S
	lr	1, A

level.loop.checkPacmanCollision.x:
	; check the X axis
	lr	A, 0
	com
	inc
	SETISAR	PACMAN_X_REG
	as	S
	bm	level.loop.checkPacmanCollision.x.mi
	; positive
	com
	inc
	ai	DEATH_DISTANCE
	bm	level.loop.checkPacmanCollision.checkLoop
	br	level.loop.checkPacmanCollision.y
	; negative
level.loop.checkPacmanCollision.x.mi
	ai	DEATH_DISTANCE
	bm	level.loop.checkPacmanCollision.checkLoop

level.loop.checkPacmanCollision.y:
	; check the Y axis
	lr	A, 1
	com
	inc
	SETISAR	PACMAN_Y_REG
	as	S
	bm	level.loop.checkPacmanCollision.y.mi
	; positive
	com
	inc
	ai	DEATH_DISTANCE
	bm	level.loop.checkPacmanCollision.checkLoop
	br	level.loop.checkPacmanCollision.collided
	; negative
level.loop.checkPacmanCollision.y.mi
	ai	DEATH_DISTANCE
	bm	level.loop.checkPacmanCollision.checkLoop

	; pacman has collided with a ghost
level.loop.checkPacmanCollision.collided:
	; end the level
	jmp	level.ended

level.loop.checkPacmanCollision.checkLoop:
	; check if we've reached the last ghost
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	inc
	ci	GHOST_NUMBER
	bz	level.loop.checkPacmanCollision.end
	; update next ghost
	lr	S, A
	jmp	level.loop.checkPacmanCollision

level.loop.checkPacmanCollision.end:

;---------------------------------------------------------------------------
; pellets
;---------------------------------------------------------------------------

    IF switch_sfx
level.loop.playPelletEffect:
	SETISAR	PACMAN_ANIMINDEX_REG
	lis	3
	xs	S						; compare with 4
	bnz	level.loop.playPelletEffectEnd

	; play dot-eating sound
	dci	sfx.pellet
	pi	playSong
level.loop.playPelletEffectEnd:
    ENDIF

level.loop.checkPellets:
	; check if pacman has eaten any pellets
	pi	pacman.checkPelletCollision

level.loop.checkBoard:
	; check to see if the board is cleared
	dci	ram

	; load our loop counter
	li	60
	lr	0, A
level.loop.checkBoard.loop:
	; decrease counter
	ds	0
	bm	level.loop.checkBoard.empty			; the board has been cleared
	; check the next byte
	lm
	ci	0
	bnz	level.loop.checkBoard.end			; there's still some pellets left, end the loop
	br	level.loop.checkBoard.loop			; continue the loop
	
level.loop.checkBoard.empty:
	; the board has been cleared
	jmp	level.cleared
level.loop.checkBoard.end:

;---------------------------------------------------------------------------
; loop end
;---------------------------------------------------------------------------

level.loop.end:
	jmp	level.loop

;===========================================================================
; Level ending functions
;===========================================================================

;---------------------------------------------------------------------------
; level cleared
;---------------------------------------------------------------------------

level.cleared:
	; draw Pac-Man's final frame
	SETISAR PACMAN_ANIMINDEX_REG
	lis	2						; load the closed-sprite index
	lr	S, A						; save the new index
	pi	pacman.draw					; redraw Pac-Man

	; load loop counter
	li	4						; how many times to flash the screen
	lr	0, A						; stored in r0

level.cleared.loop:

	; black & white
	li	$ff
	lr	1, A
	; horizontal position (250 = color row)
	li	250
	lr	2, A
	; vertical position
	li	58
	lr	3, A	
level.cleared.loop.on:
	ds	3
	bm	level.cleared.loop.on.end
	pi	plot
	br	level.cleared.loop.on
level.cleared.loop.on.end:

	; delay for a little bit
	li	$5f
	lr	5, A
	pi	delay

	; color, grey background
	li	$0
	lr	1, A
	; horizontal position (250 = color row)
	li	250
	lr	2, A
	; vertical position
	li	58
	lr	3, A
level.cleared.loop.off:
	ds	3
	bm	level.cleared.loop.off.end
	pi	plot
	br	level.cleared.loop.off
level.cleared.loop.off.end:

	; delay for a little bit
	li	$5f
	lr	5, A
	pi	delay

level.cleared.check:
	ds	0
	bnz	level.cleared.loop

	; increase the level counter
	lisu	4
	lisl	1
	lr	A, S
	inc
	lr	S, A

	; delay for a little bit
	li	$8f
	lr	5, A
	pi	delay

level.cleared.end:
	; increase the level number
	SETISAR	LEVEL_NUMBER_REG
	lr	A, S
	inc
	lr	S, A

	; start the next level
	jmp	level.init

;---------------------------------------------------------------------------
; level ended
;---------------------------------------------------------------------------

level.ended:
	; short delay
	li	$ff
	lr	5, A
	pi	delay

	; clear the ghosts
	SETISAR GHOST_NUMBER_REG
	lis	0
	lr	S, A
level.ended.clearGhosts:
	
	; move registers to temp location
	; get ISAR location for ghost
	SETISAR GHOST_NUMBER_REG
	; multiply by 4 bytes per ghost
	lr	A, S
	sl	1
	sl	1
	ai	GHOST_REG
	lr	IS, A
	lr	10, A

level.ended.clearGhosts.clear:
	; clear the ghost
	li	MAZE_BGCOLOR
	lr	1, A
	; load x coordinate
	lr	A, I
	lr	2, A
	; load y coordinate
	lr	A, D
	lr	3, A
	; set sprite index
	li	0
	lr	4, A
	; set bitmask
	li	%11111000
	lr	5, A
	; draw the sprite
	pi	sprite.draw

level.ended.clearGhosts.redrawPellets:
	; restore ISAR
	lr	A, 10
	lr	IS, A
	
	; redraw overlapped pellets
	lr	A, I
	lr	1, A
	lr	A, D
	lr	2, A
	pi	ghost.checkPelletCollision

level.ended.clearGhostscheckLoop:
	; check if we've reached the last ghost
	SETISAR GHOST_NUMBER_REG
	lr	A, S
	inc
	ci	GHOST_NUMBER
	bz	level.ended.clearGhosts.end
	; update next ghost
	lr	S, A
	br	level.ended.clearGhosts
level.ended.clearGhosts.end:

	; start the death sequence
level.ended.deathSequence:
	; load pacman's coordinates
	SETISAR	PACMAN_X_REG
	; x register
	lr	A, I
	lr	2, A
	; y register
	lr	A, I
	lr	3, A

	; part 1
level.ended.deathSequence.1:
	; clear Pac-Man
	pi	pacman.clear

	; load position
	SETISAR	PACMAN_X_REG
	lr	A, I
	lr	2, A
	lr	A, S
	lr	3, A
	; load color
	li	PACMAN_COLOR
	lr	1, A
	; sprite index
	lis	1
	lr	4, A
	; draw the sprite
	pi	sprite.draw				; redraw Pac-Man

    IF switch_music
	; play death tune
	dci	sfx.dying.0
	pi	playSong
    ENDIF
   
	; part 2
level.ended.deathSequence.2:
	; clear Pac-Man
	pi	pacman.clear

	; load position
	SETISAR	PACMAN_X_REG
	lr	A, I
	lr	2, A
	lr	A, S
	lr	3, A
	; load color
	li	PACMAN_COLOR
	lr	1, A
	; load the sprite index
	li	20
	lr	4, A
	; draw the sprite
	pi	sprite.draw				; redraw Pac-Man

    IF switch_music
	; play death tune
	dci	sfx.dying.1
	pi	playSong
    ENDIF

	; part 3
level.ended.deathSequence.3:
	; clear Pac-Man
	pi	pacman.clear

	; load position
	SETISAR	PACMAN_X_REG
	lr	A, I
	lr	2, A
	lr	A, S
	lr	3, A
	; load color
	li	PACMAN_COLOR
	lr	1, A
	; load the sprite index
	li	21
	lr	4, A
	; draw the sprite
	pi	sprite.draw				; redraw Pac-Man

    IF switch_music
	; play death tune
	dci	sfx.dying.2
	pi	playSong
    ENDIF
   
   	; part 4
level.ended.deathSequence.4:
	; clear Pac-Man
	pi	pacman.clear

	; load position
	SETISAR	PACMAN_X_REG
	lr	A, I
	lr	2, A
	lr	A, S
	lr	3, A
	; load color
	li	PACMAN_COLOR
	lr	1, A
	; load the sprite index
	li	22
	lr	4, A
	; draw the sprite
	pi	sprite.draw				; redraw Pac-Man

    IF switch_music
	; play death tune
	dci	sfx.dying.3
	pi	playSong
    ENDIF

   	; part 5
level.ended.deathSequence.5:
	; clear Pac-Man
	pi	pacman.clear

level.ended.restart:
	; delay for a little bit
	li	$ff
	lr	5, A
	pi	delay

	; decrease Pac-Man's life counter and
	; reset the level if he has lives remaining
	SETISAR	PACMAN_LIVES_REG
	ds	S
	bz	level.ended.reset
	
	; continue the level
	jmp	level.init.intro

level.ended.reset:
	; check if we're going to set a new high score
	SETISAR	HISCORE_REG
	; compare 1st digits
	lr	A, S
	com
	inc
	lisl	4
	as	S
	bm	level.ended.reset.updateHS.end			; if negative, no high score
	bnz	level.ended.reset.updateHS			; if positive, we have a new HS
	; compare 2nd digits
	lisl	1
	lr	A, S
	com
	inc
	lisl	5
	as	S
	bm	level.ended.reset.updateHS.end			; if negative, no high score
	bnz	level.ended.reset.updateHS			; if positive, we have a new HS
	; compare 3rd digits
	lisl	2
	lr	A, S
	com
	inc
	lisl	6
	as	S
	bm	level.ended.reset.updateHS.end			; if negative, no high score
	bnz	level.ended.reset.updateHS			; if positive, we have a new HS
	; compare 4th digits
	lisl	3
	lr	A, S
	com
	inc
	lisl	7
	as	S
	bm	level.ended.reset.updateHS.end			; if negative, no high score
	
level.ended.reset.updateHS:
	; update the high score
	lisl	4
	lr	A, S
	lisl	0
	lr	S, A
	lisl	5
	lr	A, S
	lisl	1
	lr	S, A
	lisl	6
	lr	A, S
	lisl	2
	lr	S, A
	lisl	7
	lr	A, S
	lisl	3
	lr	S, A
	
	; draw the new hiscore
	pi	hiscore.draw
level.ended.reset.updateHS.end:

;[TODO] gameover screen? jump back to title screen?

	; clear the score
	SETISAR	SCORE_REG
	clr
	lr	I, A
	lr	I, A
	lr	I, A
	lr	S, A
	
	; set pac-man lives
	; load pacman's number of lives

	SETISAR	PACMAN_LIVES_REG
	lis	PACMAN_LIVES
	lr	S, A
	

	; start a new level
	jmp	level.init

;===========================================================================
; Pellet Functions 
;===========================================================================

;---------------------------------------------------------------------------
; Pellet Initalization
;---------------------------------------------------------------------------
; copy the pellet map into memory

pellets.init:
	; load DC1 with the RAM address
	dci	ram
	xdc
	; load DC0 with the map address
	dci	maze.pellets

	; start the copy cycle
	li	60
	lr	0, A
pellets.init.loop:
	; get and save a byte
	lm
	xdc
	st
	xdc

	; check if the loop has ended
	ds	0
	bnz	pellets.init.loop

pellets.init.end:
	pop


;---------------------------------------------------------------------------
; Power Pellet Initalization
;---------------------------------------------------------------------------
; copy the power pellet map into memory

powerPellets.init:
	; load DC1 with the RAM address
	dci	ram+$40
	xdc
	; load DC0 with the map address
	dci	maze_powerPellets

	; start the copy cycle
	li	60
	lr	0, A
powerPellets.init.loop:
	; get and save a byte
	lm
	xdc
	st
	xdc

	; check if the loop has ended
	ds	0
	bnz	powerPellets.init.loop

powerPellets.init.end:
	pop

;---------------------------------------------------------------------------
; Check Pacman's Intersection 
;---------------------------------------------------------------------------
; checks if pac-man is in an intersection

pacman.checkIntersection:
	SETISAR	PACMAN_X_REG
	; X position
	lr	A, I
	lr	4, A
	; Y position
	lr	A, S
	lr	5, A
	; jump to checkIntersection
	br	checkIntersection

;---------------------------------------------------------------------------
; Check Intersection 
;---------------------------------------------------------------------------
; checks to see if a location is in an intersection (r4 = x, r5 = y)
; if so, it returns with A set to 1 and the intersection boundaries in r0
; else, returns with A set to 0
;
; modifies: ISAR, r0, r1, r2, r3, Q, K
;
; MISERABLY unoptimized, I'll need to fix that sometime...

checkIntersection:
	; save the return address
	lr	K, P
	pi	kstack.push

	; check to see if we're on an intersection horizontally
	lr	A, 4
.checkIntersectionHorizLoop:
	ai	%11111010					; keep subtracting 6
	bz	.checkIntersectionHorizLoopEnd			; if it's 0, we're on an intersection horizontally
	bnc	.checkIntersectionFalse				; if it's < 6, break
	br	.checkIntersectionHorizLoop			; continue the loop
.checkIntersectionHorizLoopEnd:

	; check to see if we're on an intersection vertically
	lr	A, 5
	ai	7						; fix -1 offset of intersections
.checkIntersectionVerticLoop:
	ai	%11111010					; keep subtracting 6
	bz	.checkIntersectionTrue				; if it's 0, we're on an intersection vertically
	bnc	.checkIntersectionFalse				; if it's < 6, break
	br	.checkIntersectionVerticLoop			; continue the loop

.checkIntersectionFalse:
	; return from the subroutine
	pi	kstack.pop
	lis	0						; not on an intersection, return false
	pk

	; get the intersection boundaries
.checkIntersectionTrue:
	; load the map address
	dci	maze.map

	; add y offset to DC
	li	$ff						; 0, complemented
	lr	0, A						; r0 will be our offset	
	; get the y address
	lr	A, 5
	inc							; fix -1 offset of intersections
.checkIntersectionYLoop:
	ai	%11111010					; keep subtracting 6
	bnc	.checkIntersectionYLoopEnd			; if it's < 0, then we're done
	ds	0						; "incrase" r0
	br	.checkIntersectionYLoop
.checkIntersectionYLoopEnd:
	; get the true offset
	lr	A, 0
	com							; invert r0
	lr	0, A
	lr	2, A						; save Y position
	li	12
.checkIntersectionYLoop2:
	ds	0
	bnc	.checkIntersectionYLoop2End
	adc							; add 24 bytes to the DC
	br	.checkIntersectionYLoop2			; repeat
.checkIntersectionYLoop2End:

	; add x offset to DC
	li	$ff						; 0, complemented
	lr	0, A						; r0 will be our offset	
	; get the x address
	lr	A, 4
	ai	%11111010					; subtract the initial 6
.checkIntersectionXLoop:
	ai	%11111010					; keep subtracting 6
	bnc	.checkIntersectionXLoopEnd			; if it's < 0, then we're done
	ds	0						; "incrase" r0
	br	.checkIntersectionXLoop
.checkIntersectionXLoopEnd:
	; add the offset
	lr	A, 0
	com							; invert r0 to get the offset
	adc
	lr	3, A						; save X position

	; save the address
	lr	Q, DC

	; now we find this tile's boundaries	
	lm							; get the tile number
	lr	1, A						; save the tile number
	pi	checkIntersection.getBoundary			; find the boundaries
	lr	0, A						; and save those

	; find the right tile's left boundary,
	; mask it, and AND it to this one's
.checkRightBoundary:
	; check to see if we're on the edge
	lis	%1011
	xs	3
	bnz	.checkRightBoundaryTile				; not on the edge, get the boundary

	; mask off right boundary
	li	%00001110
	ns	0
	lr	0, A
	br	.checkRightBoundaryEnd

.checkRightBoundaryTile:
	; first, fix the address
	lr	DC, Q						; retrieve the address
	lm
	; retrieve the tile address
	lm							; get the tile number
	lr	1, A						; save the tile number
	pi	checkIntersection.getBoundary			; find the boundaries
	ni	%00000010					; mask it
	sr	1
	ai	%00001110					; set the other bits
	ns	0						; and AND it with this boundary
	lr	0, A						; finally, save it
.checkRightBoundaryEnd:	

	; find the bottom tile's upper boundary,
	; mask it, and AND it to this one's
.checkBottomBoundary:
	; check to see if we're on the edge
	lis	%1001
	xs	2
	bnz	.checkBottomBoundaryTile			; not on the edge, get the boundary

	; mask off bottom boundary
	li	%00001011
	ns	0
	lr	0, A
	br	.checkBottomBoundaryEnd

.checkBottomBoundaryTile:
	; first, fix the address
	lr	DC, Q						; retrieve the address
	lis	12
	adc
	; now, get the byte and work with it
	lm							; get the tile number
	lr	1, A						; save the tile number
	pi	checkIntersection.getBoundary			; find the boundaries
	ni	%00001000					; mask it
	sr	1
	ai	%00001011					; set the other bits
	ns	0						; and AND it with this boundary
	lr	0, A						; finally, save it	
.checkBottomBoundaryEnd:

.checkIntersectionEnd:
	; return from the subroutine
	pi	kstack.pop
	lis	1						; return true
	pk

; sub-subroutine, where we find the boundaries of a tile number

checkIntersection.getBoundary:
	; load the tile address into the DC
	dci	tiles
	lis	3						; three bytes for each tile
.checkIntersection.getBoundaryAddressLoop:
	ds	1
	bnc	.checkIntersection.getBoundaryAddressLoopEnd
	adc
	br	.checkIntersection.getBoundaryAddressLoop
.checkIntersection.getBoundaryAddressLoopEnd:
	lis	2
	adc
	lm
	; return
	pop

;---------------------------------------------------------------------------
; Check Ghost Pellet Collision
;---------------------------------------------------------------------------
; checks to see if a ghost is overlapping a pellet, and if that pellet is
; still turned on, redraws it

ghost.checkPelletCollision:
	; save return address
	lr	K, P
	pi	kstack.push

	; set the callback function
	li	>ghost.checkPelletCollision.callback
	lr	Ku, A
	li	<ghost.checkPelletCollision.callback
	lr	Kl, A
	
	; check if pacman has collided with a pellet
	jmp	pellets.checkCollision

; callback for the pellets.checkCollision routine, that redraws a pellet
; onscreen if it is still turned on, using the bit specified in r0 and the
; address at DC0

ghost.checkPelletCollision.callback:
	; save return address
	lr	K, P
	pi	kstack.push
	
	; load the plot parameters
	li	PELLET_COLOR
	lr	1, A
	lr	A, 7
	lr	2, A
	lr	A, 8
	lr	3, A
	; plot the pellet
	pi	plot
	
ghost.checkPelletCollision.callback.end:
	; retrieve the return address, store in P
	pi	kstack.pop
	lr	P, K
	
	; restore the callback location
	li	>ghost.checkPelletCollision.callback
	lr	Ku, A
	li	<ghost.checkPelletCollision.callback
	lr	Kl, A
	
	; return
	pop

;---------------------------------------------------------------------------
; Check Pacman Pellet Collision
;---------------------------------------------------------------------------
; checks to see if pac-man is overlapping a pellet, and if so, turns off its
; respective bit in RAM and increases the score

pacman.checkPelletCollision:
	; save return address
	lr	K, P
	pi	kstack.push

	; set the callback function
	li	>pacman.checkPelletCollision.callback
	lr	Ku, A
	li	<pacman.checkPelletCollision.callback
	lr	Kl, A

	; load the coordinates
	SETISAR	PACMAN_X_REG
	lr	A, I
	lr	1, A						; X position
	lr	A, S
	lr	2, A						; Y position
	
	; check if pacman has collided with a pellet
	jmp	pellets.checkCollision

; callback for the pellets.checkCollision routine, to turn off the pellet's
; respective bit in RAM. takes the bit specified in r0 and masks the bit in
; the address at DC0

pacman.checkPelletCollision.callback:
	; save return address
	lr	K, P
	pi	kstack.push
	
	; load the bitmask
	lr	A, 4
	com
	; mask the byte
	lr	DC, H
	nm
	; and store it
	lr	DC, H
	st
	
	; add to the score
	li	%01100110
	lr	Ku, A
	lr	Kl, A
	lr	Qu, A
	li	%01110110
	lr	Ql, A						; 10 points
	pi	score.add
	
	; output dot-eating sound
	pi	sfx.dotEating.play
	
pacman.checkPelletCollision.callback.checkBonus:
	; check if a bonus should be awarded (>=10,000 points)
	SETISAR SCORE_REG + 2

	; compare third digit of score (00)
	lr	A, I
	ci	$00
	bnz	pacman.checkPelletCollision.callback.end
	; compare fourth digit of score (00)
	lr	A, I
	ci	$00
	bnz	pacman.checkPelletCollision.callback.end
	
pacman.checkPelletCollision.callback.checkBonus.award:
	; give Pac-Man an extra life
	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	inc
	lr	S, A
    IF switch_music
	; play the extra life sound
	dci	sfx.extraPacman
	pi	playSong
    ENDIF

	; redraw the lives display
	pi	pacman.drawLives

pacman.checkPelletCollision.callback.end:
	; retrieve the return address, store in P
	pi	kstack.pop
	lr	P, K
	
	li	>pacman.checkPelletCollision.callback
	lr	Ku, A
	li	<pacman.checkPelletCollision.callback
	lr	Kl, A
	pop


;---------------------------------------------------------------------------
; Pellet Check 
;---------------------------------------------------------------------------
; this function checks to see if an object at the coordinates in (r0, r1) is
; overlapping a pellet. this function should not be called directly.
;
; modifies: r0-2, r4-r8, H, K, Q, DC
; r0 = temp
; r1 = initial X position
; r2 = initial Y position
; r3 = [unused]
; r4 = bit counter
; r5 = Pac-Man's X offset
; r6 = Pac-Man's Y offset
; r7 = Pellet's X position
; r8 = Pellet's Y position

pellets.checkCollision:
	; initialize the data pointer
	dci	ram	
	; initialize the bitmask
	li	%10000000
	lr	4, A

	; find the X position
	li	$fd						; -3
	lr	0, A						; store in r0 for easy subtraction
	; load the X position and adjust it
	lr	A, 1						; load Pac-Man's X position
	ai	$fb						; -5
pellets.checkCollision.XOffset:
	; save the offset
	lr	5, A
	; check if the offset is negative or 0
	ci	0
	bp	pellets.checkCollision.XOffset.end

	; shift the bitmask right
	lr	A, 4
	sr	1
	bnz	pellets.checkCollision.XOffset.noPointerAdjust
	; carry the bitmask
	lm							; increase DC0
	li	%10000000
pellets.checkCollision.XOffset.noPointerAdjust:
	lr	4, A
	; subtract 3 from the position
	lr	A, 5
	as	0
	; loop
	br	pellets.checkCollision.XOffset
pellets.checkCollision.XOffset.end:
	; find the UL pellet's coordinate
	com
	inc
	as	1
	lr	7, A

	; load the Y coordinate and adjust it
	lis	2
	as	2						; load Pac-Man's Y position
pellets.checkCollision.YOffset:
	; save the offset
	lr	6, A
	; check if the offset is negative or 0
	ci	0
	bp	pellets.checkCollision.YOffset.end

	; increment the DC0 by 3
	lis	3
	adc
	; subtract 3 from the position
	lr	A, 6
	as	0
	; loop
	br	pellets.checkCollision.YOffset
pellets.checkCollision.YOffset.end:
	; find the UL pellet's coordinate
	com
	inc
	as	2
	lr	8, A

	; preserve the DC
	lr	H, DC

pellets.checkCollision.overlap.tl:
	; check that this pellet is on
	lm
	ns	4
	bz	pellets.checkCollision.overlap.tr
	; call the callback
	pk

pellets.checkCollision.overlap.tr:
	; adjust the memory pointer
	lr	DC, H						; load the preserved DC0
	lr	A, 4
	sr	1
	bnz	pellets.checkCollision.overlap.tr.noPointerAdjust
	; overflow
	lm							; increase DC0
	lr	H, DC						; and preserve it
	li	%10000000					; roll the bitmask over
pellets.checkCollision.overlap.tr.noPointerAdjust:
	lr	4, A
	; adjust pellet coordinate
	lis	3
	as	7
	lr	7, A

	; check that we're overlapping this pixel
	lis	1
	as	5
	bm	pellets.checkCollision.overlap.br		; if offsetX < -1, no overlap
	; check that this pellet is on
	lm
	ns	4
	bz	pellets.checkCollision.overlap.br

	; call the callback
	pk

pellets.checkCollision.overlap.br:
	; adjust the memory pointer
	lr	DC, H						; load the preserved DC0
	lis	3						; 24 pellets, one row
	adc							; increment the pointer
	lr	H, DC						; preserve it
	; adjust pellet coordinate
	lis	3
	as	8
	lr	8, A

	; check that we're overlapping this pixel
	lis	1
	as	5
	bm	pellets.checkCollision.overlap.bl		; if offsetX < -1, no overlap
	; check that we're overlapping this pixel
	lis	1
	as	6
	bm	pellets.checkCollision.end			; if offsetY < -1, no overlap
	; check that this pellet is on
	lm
	ns	4
	bz	pellets.checkCollision.overlap.bl

	; call the callback
	pk

pellets.checkCollision.overlap.bl:
	; adjust the memory pointer
	lr	DC, H						; load the preserved DC0
	lr	A, 4
	sl	1
	bnz	pellets.checkCollision.overlap.bl.noPointerAdjust
	; overflow
	ds	Hl						; decrease the stored DC0
	lr	DC, H						; and load it
	lis	%0001						; roll the bitmask over
pellets.checkCollision.overlap.bl.noPointerAdjust:
	lr	4, A
	; adjust pellet coordinate
	ds	7
	ds	7
	ds	7

	; check that we're overlapping this pixel
	lis	1
	as	6
	bm	pellets.checkCollision.end			; if offsetY < -1, no overlap
	; check that this pellet is on
	lm
	ns	4
	bz	pellets.checkCollision.end

	; call the callback
	pk
	
pellets.checkCollision.end:
	; return
	pi	kstack.pop
	pk

;---------------------------------------------------------------------------
; Powerpellet Check 
;---------------------------------------------------------------------------
; this function checks to see if an object at the coordinates in (r0, r1) is
; overlapping a powerpellet. this function should not be called directly.
;
; modifies: r1-4
; r1 = Pellet's X position
; r2 = Pellet's Y position
; r3 = initial X position
; r4 = initial Y position

powerPellets.checkCollision:
	; save return address
	lr	K, P
	pi	kstack.push

	; copy the registers
	lr	A, 1
	lr	4, A
	lr	A, 2
	lr	5, A
	
	; check coordinates
powerPellets.checkCollision.x1:
	; check that the x value is in range in left column
	lr	A, 4
	ci	2
	bp	powerPellets.checkCollision.x2
	ci	9
	bm	powerPellets.checkCollision.x2
	; load the pellet's coordinate
	lis	7
	lr	2, A
	br	powerPellets.checkCollision.y1
powerPellets.checkCollision.x2
	; check that the x value is in range in right column
	lr	A, 4
	ci	68
	bp	powerPellets.checkCollision.end
	ci	75
	bm	powerPellets.checkCollision.end
	; load the pellet's coordinate
	li	73
	lr	2, A
	br	powerPellets.checkCollision.y1
	
powerPellets.checkCollision.y1:
	; check that the y value is in range in top row
	lr	A, 5
	ci	%11111110					; -2
	bp	powerPellets.checkCollision.y2
	ci	5
	bm	powerPellets.checkCollision.y2
	; load the pellet's coordinate
	lis	3
	lr	3, A
	br	powerPellets.checkCollision.collided
powerPellets.checkCollision.y2
	; check that the y value is in range in bottom row
	lr	A, 5
	ci	37
	bp	powerPellets.checkCollision.end
	ci	44
	bm	powerPellets.checkCollision.end
	; load the pellet's coordinate
	li	42
	lr	3, A

powerPellets.checkCollision.collided:
	; the object has collided with a powerpellet, redraw it
	li	POWERPELLET_COLOR
	lr	1, A
	; load the sprite number
	li	23
	lr	4, A	
	; draw the pellet
	pi	sprite.draw	
	
powerPellets.checkCollision.end:
	; return
	pi	kstack.pop
	pk

;===========================================================================
; Score Functions 
;===========================================================================

;---------------------------------------------------------------------------
; Add score 
;---------------------------------------------------------------------------

; adds a 4-byte decimal adjusted number to the score
; stored in r12-r15 (K, Q)

score.add:
	; set the ISAR
	SETISAR [SCORE_REG + 3]

	; non-carry loop
score.add.loop:
	; check to see if the loop is ended
	lr	A, IS
	ci	[SCORE_REG - 1]
	bz	score.add.end					; loop is ended, break

score.add.loop.addByte:
	lisu	1
	lr	A, S
	lisu	[SCORE_REG >> 3]
	asd	S						; add two decimal-adjusted digits
	lr	D, A						; save digit and decrease ISAR

	bc	score.add.carryLoop				; we've carried over, use the carry loop
	br	score.add.loop					; loop

	; carry loop
score.add.carryLoop:
	; check to see if the loop is ended
	lr	A, IS
	ci	[SCORE_REG - 1]
	bz	score.add.end					; loop is ended, break

	; add carry
	lis	%0111						; 1 in BCD
	asd	S
	lr	S, A						; save digit in score
	bnc	score.add.loop.addByte				; no carry, use the first loop

score.add.carryLoop.addByte:
	lisu	1
	lr	A, S
	lisu	[SCORE_REG >> 3]
	asd	S						; add two decimal-adjusted digits
	lr	D, A						; save digit in score and decrease ISAR

	br	score.add.carryLoop				; loop

score.add.end:
	; return from the subroutine
	pop

;===========================================================================
; Drawing Functions 
;===========================================================================

	; basic drawing functions
	include "drawing.inc"

;---------------------------------------------------------------------------
; Draw Tile 
;---------------------------------------------------------------------------
; draw an 8x8 tile from a data pointer
; r1 = color 1 (off)
; r2 = color 2 (on)
; r3 = x position
; r4 = y position
; r5 = tile number
;
; modifies: r1-r9 (blit), Q, DC

drawTile:
	; get the tile address
	dci	tiles
	; add the offset
	lr	A, 5
	inc							; make sure we hit 0
	lr	0, A
	lis	3						; three bytes for each tile
.drawTileAddressLoop:
	ds	0
	bz	.drawTileAddressLoopEnd
	adc
	br	.drawTileAddressLoop
.drawTileAddressLoopEnd:
	; got the tile data, now get the graphics address
	lm
	lr	Qu, A
	lm
	lr	Ql, A
	lr	DC, Q

	; load the width and height
	li	6
	lr	5, A
	lr	6, A

.drawTileBlit:
	; draw the tile itself
	jmp	blit

;---------------------------------------------------------------------------
; Draw Map 
;---------------------------------------------------------------------------
; draw the pac-man maze in two colors
; r0 = loop counter
; r1-r9 = routine arguments
; r10 = x counter
; r11 = y counter

map.draw:
	; load the return address into memory
	lr	K, P
	pi	kstack.push

	; load the map address into DC1
	dci	maze.map
	xdc
	
	; load the initial X and Y positions of the tiles
	lis	5
	lr	3, A
	li	$fe						; -2
	lr	4, A

	; load r10 with 10 rows of tiles
	lis	10
	lr	10, A
map.draw.loop.row:

	; load r11 with 12 columns of tiles
	lis	12
	lr	11, A
map.draw.loop.column:

	; note of reference to drawTile:
	; r1 = color 1
	; r2 = color 2
	; r3 = x (to screen)
	; r4 = y (to screen)
	; r5 = tile number

map.draw.loop.drawTile:

	; this block of code gets the next tile number
	; and increases the data offset

	; swap DC0 and DC1
	xdc
	; get and save the next tile byte
	lm
	lr	5, A
	; swap back
	xdc

	; set the colors
	; color 1
	li	MAZE_BGCOLOR
	lr	1, A						; r1
	; color 2
	li	MAZE_FGCOLOR
	lr	2, A						; r2

	; finally, call the tile function
	pi	drawTile
	
	; drawTile adds 4 to both the X and Y positions

map.draw.loop.column.end:
	; increase the tile X position
	lis	2
	as	3
	lr	3, A
	; reset the tile Y position
	li	$fc
	as	4
	lr	4, A
	; decrement column counter
	ds	11
	bnz	map.draw.loop.column
	
map.draw.loop.row.end:
	; reset the tile X position
	lis	5
	lr	3, A
	; increase the tile Y position
	lis	6
	as	4
	lr	4, A
	; decrement row counter
	ds	10
	bnz	map.draw.loop.row

map.draw.drawGhostGate:
	; draw the exit gate for the ghost
	li	GATE_COLOR
	lr	1,A
	li	39
	lr	2,A
	li	22
	lr	3,A
	li	19
	lr	4,A
	li	%11111000
	lr	5,A
	pi	sprite.draw	
	
map.draw.end:
	; return to the program flow
	pi	kstack.pop
	pk							; return from the subroutine

;---------------------------------------------------------------------------
; Draw Sprite 
;---------------------------------------------------------------------------
; draw a 8x5 sprite from a data pointer
; r1 = color
; r2 = x (to screen)
; r3 = y (to screen)
; r4 = sprite number

sprite.draw:
	; blit reference:
	; r1 = color 1 (off)
	; r2 = color 2 (on)
	; r3 = x position
	; r4 = y position
	; r5 = width
	; r6 = height

	; get the tile address
	dci	sprites
	; add the offset
	lr	A, 4
	inc							; make sure we hit 0
	lr	0, A
	lis	2						; two bytes for each sprite
sprite.draw.addressLoop:
	ds	0
	bz	sprite.draw.addressLoop.end
	adc
	br	sprite.draw.addressLoop
sprite.draw.addressLoop.end:
	; load the address for this sprite's number
	lm
	lr	Qu, A
	lm
	lr	Ql, A
	lr	DC, Q
	
	; load the sprite size (8x5)
	lis	8
	lr	5, A
	lis	5
	lr	6, A
	; load the position
	lr	A, 3
	lr	4, A
	lr	A, 2
	lr	3, A
	; load the colors
	lr	A, 1
	lr	2, A
	li	clear
	lr	1, A

	; draw the sprite
	jmp	blit
	
;---------------------------------------------------------------------------
; Draw Sidebar 
;---------------------------------------------------------------------------

; draws the right sidebar

sidebar.draw:
	; save return address
	lr	K, P
	pi	kstack.push

	; set sidebar color
	li	SIDEBAR_COLOR
	lr	1, A

	; start row loop
	li	57
	lr	3, A
sidebar.draw.row:
	li	97
	lr	2, A
sidebar.draw.column:
	pi	plot

sidebar.drawCheck:
	; check column
	ds	2
	lr	A, 2
	ci	76						; see if we've reached the left edge
	bnz	sidebar.draw.column

	; check row
	ds	3
	bc	sidebar.draw.row					; branch if > 0

sidebar.drawText:
	; draw "SCORE"
	dci	gfx.score.parameters
	pi	blitGraphic
	; draw "HIGH"
	dci	gfx.high.parameters
	pi	blitGraphic

sidebar.drawEnd:
	; return from subroutine
	pi	kstack.pop
	pk

;---------------------------------------------------------------------------
; Draw Hi-Score 
;---------------------------------------------------------------------------
; draws the 8-digit hi-score to the screen

hiscore.draw:
	; save return address
	lr	K, P
	pi	kstack.push

	; load colors
	li	SIDEBAR_COLOR
	lr	1, A
	li	SCORE_COLOR
	lr	2, A

	; load position
	li	78
	lr	10, A
	li	24
	lr	11, A

	; set the number counter (to detect when to wrap)
	lis	8
	lr	0, A
	; set ISAR
	SETISAR HISCORE_REG
	
	; branch to draw score
	br	score.draw.loop

;---------------------------------------------------------------------------
; Draw Score 
;---------------------------------------------------------------------------
; draws the 8-digit score to the screen

score.draw:
	; save return address
	lr	K, P
	pi	kstack.push

	; load colors
	li	SIDEBAR_COLOR
	lr	1, A
	li	SCORE_COLOR
	lr	2, A

	; load position
	li	78
	lr	10, A
	lis	6
	lr	11, A

	; set the number counter (to detect when to wrap)
	lis	8
	lr	0, A
	; set ISAR
	SETISAR SCORE_REG

score.draw.loop:
	; set DC
	dci	gfx.numbers

	; load size
	lis	3
	lr	5, A
	lis	5
	lr	6, A

	; load position
	lr	A, 10
	lr	3, A
	lr	A, 11
	lr	4, A

	; see if the tile is even or odd
	lis	%1
	ns	0
	bz	score.draw.evenByte				; if even, branch

score.draw.oddByte:
	; get lower half-byte
	lis	%1111
	ns	I						; set ISAR to next byte
	br	score.draw.drawDigit
score.draw.evenByte:
	; get upper half-byte
	lr	A, S
	sr	4

score.draw.drawDigit:
	; add tile offset
	sl	1
	adc
	; and draw the number
	pi	blit

	; increase x position
	lis	4
	as	10
	lr	10, A						; each tile is three pixels + one pixel space

	; decrease the counter
	ds	0
	bz	score.draw.end					; reached the end, break
	; check to see if we're on tile 4
	lis	4
	xs	0
	bnz	score.draw.loop

score.draw.changeRow:
	; increase X position
	li	$f4
	as	10
	lr	10, A
	; increase Y position
	lis	6
	as	11
	lr	11, A
	br	score.draw.loop

score.draw.end:
	; return from subroutine
	pi	kstack.pop
	pk

;===========================================================================
; Object Drawing Functions 
;===========================================================================

;---------------------------------------------------------------------------
; Draw Pellets 
;---------------------------------------------------------------------------
; this function draws out the pellets as specified by "maze.pellets" and
; "maze_powerPellets"

pellets.draw:
	lr	K, P
	pi	kstack.push					; save return address

	; load our data pointer
	dci	ram

	; load our color
	li	PELLET_COLOR
	lr	1, A

	; set the initial coordinates
	lis	5						; 5
	lr	2, A						; x coordinate
	li	%11111110					; -2
	lr	3, A						; y coordinate

	; load our byte loop counter
	li	60
	lr	4, A
pellets.draw.byteLoop:

	; load next byte
	lm
	lr	6, A

	; load our bit mask
	li	%10000000
	lr	5, A
pellets.draw.bitLoop:
	; see if there's a pellet here
	lr	A, 6
	ns	5
	; if not, branch
	bz	pellets.draw.bitLoopEnd

	; plot the pixel
	pi	plot
	
pellets.draw.bitLoopEnd:
	; add three to the x coordinate
	lr	A, 2
	ai	3						; add three pixels
	lr	2, A						; store it back in
	; if we've reach the end, roll over
	ci	77
	bnz	pellets.draw.loopCheck

	; reset x, add 3 to y
	li	5
	lr	2, A
	lr	A, 3
	ai	3						; add three pixels
	lr	3, A						; store it back in the y coordinate

pellets.draw.loopCheck:
	; check bit loop
	lr	A, 5
	sr	1
	lr	5, A
	bnz	pellets.draw.bitLoop

	; check byte loop
	ds	4
	bnz	pellets.draw.byteLoop

pellets.draw.end:
	; return from the subroutine
	pi	kstack.pop
	pk

;---------------------------------------------------------------------------
; Draw Power Pellets 
;---------------------------------------------------------------------------
; this function draws out the power pellets 

; 1:(7,3)  
; 2:(73,3)
; 3:(73,42)
; 4:(7,42)

powerPellets.draw:
	; save return address
	lr	K, P
	pi	kstack.push

powerPellets.draw.0:
	; check if this pellet is on
	dci	ram + 6
	lm
	ni	%01000000
	bz	powerPellets.draw.1

	; load the color
	li	POWERPELLET_COLOR
	lr	1, A
	; load x coordinate
	lis	7
	lr	2, A
	; load y coordinate
	lis	3
	lr	3, A
	; load the sprite number
	li	23
	lr	4, A
	
	; draw the pellet
	pi	sprite.draw
	
powerPellets.draw.1:
	; check if this pellet is on
	dci	ram + 8
	lm
	ni	%00000001
	bz	powerPellets.draw.2

	; load the color
	li	POWERPELLET_COLOR
	lr	1, A
	; load x coordinate
	li	73
	lr	2, A
	; load y coordinate
	lis	3
	lr	3, A
	; load the sprite number
	li	23
	lr	4, A
	
	; draw the pellet
	pi	sprite.draw

powerPellets.draw.2:
	; check if this pellet is on
	dci	ram + 45
	lm
	ni	%01000000
	bz	powerPellets.draw.3

	; load the color
	li	POWERPELLET_COLOR
	lr	1, A
	; load x coordinate
	lis	7
	lr	2, A
	; load y coordinate
	li	42
	lr	3, A
	; load the sprite number
	li	23
	lr	4, A
	
	; draw the pellet
	pi	sprite.draw

powerPellets.draw.3:
	; check if this pellet is on
	dci	ram + 47
	lm
	ni	%00000001
	bz	powerPellets.draw.end

	; load the color
	li	POWERPELLET_COLOR
	lr	1, A
	; load x coordinate
	li	73
	lr	2, A
	; load y coordinate
	li	42
	lr	3, A
	; load the sprite number
	li	23
	lr	4, A
	
	; draw the pellet
	pi	sprite.draw

powerPellets.draw.end:
	; return from the subroutine
	pi	kstack.pop
	pk

;---------------------------------------------------------------------------
; Clear Pac-Man 
;---------------------------------------------------------------------------
; erased Pac-Man to the background color

pacman.clear:
	; load the maze's bgcolor
	li	MAZE_BGCOLOR
	lr	1, A
	SETISAR	PACMAN_X_REG
	; load x coordinate
	lr	A, I
	lr	2, A
	; load y coordinate
	lr	A, S
	lr	3, A
	; load sprite number (0, clear)
	lis	0
	lr	4, A
	; draw the sprite
	jmp	sprite.draw

;===========================================================================
; Pac-Man Functions 
;===========================================================================

;---------------------------------------------------------------------------
; Draw Pac-Man 
;---------------------------------------------------------------------------
; draws pac-man based on the data in all the registers

pacman.draw:
	; save return address
	lr	K, P
	pi	kstack.push

	; load Pac-Man's color
	li	PACMAN_COLOR
	lr	1, A

pacman.draw.init:
	; load Pac-Man's direction
	SETISAR	PACMAN_DIRECTION_REG
	lr	A, S						; r37

pacman.draw.isDirectionDown:
	ci	%00000100					; is it down?
	bm	pacman.draw.directionUp				; negative result means up
	bnz	pacman.draw.isDirectionRight			; if not zero check if down
	lis	5						; set index to #5
	br	pacman.draw.directionEnd

pacman.draw.isDirectionRight:
	ci	%00000001					; is it right?
	bnz	pacman.draw.directionLeft			; if not direction is left
	lis	13						; set index to #13
	br	pacman.draw.directionEnd   

pacman.draw.directionUp:
	lis	1						; set index to #1
	br	pacman.draw.directionEnd

pacman.draw.directionLeft:
	lis	9						; set index to #9

pacman.draw.directionEnd:
	lr	4, A						; store index in r4

	; find the sprite's animation offset
	SETISAR	PACMAN_ANIMINDEX_REG
	lr	A, S						; load the animation index
	lr	0, A						; r0 is our loop counter
	lr	A, 4						; get back the original index
pacman.draw.getOffset:
	ds	0
	bm	pacman.draw.getOffsetEnd
	inc							; use the next tile
	br	pacman.draw.getOffset
pacman.draw.getOffsetEnd:
	lr	4, A						; set the index again

pacman.draw.warp:
	; set preliminary bitmask for when we're not looping
	li	%11111000
	lr	5, A

	SETISAR	PACMAN_X_REG
	lr	A, S
	ci	128						; check if it's off the left side
	bp	pacman.draw.warpCheckRight			; if not, check the right

	; now we find the bitmask for pacman
	com							; invert the position and increase it
	inc							; to get the number of bits to be masked
	lr	0, A						; save it
	li	%11111000					; load our new bitmask
pacman.draw.warpLeftLoop:
	ds	0
	bm	pacman.draw.warpLeftLoopEnd			; if we've reached the end of the bits, end the loop
	sr	1
	br	pacman.draw.warpLeftLoop
pacman.draw.warpLeftLoopEnd:
	lr	5, A						; save the bitmask
	br	pacman.draw.warpEnd				; end the warping check

pacman.draw.warpCheckRight:
	lr	A, S
	ci	75						; check if it's off the right side
	bp	pacman.draw.warpEnd				; if not, end the loop

	; now we find the bitmask for pacman
	ai	%10110101					; subtract 75 pixels to get the number of bits to be masked
	lr	0, A						; save it
	li	%11111000					; load our new bitmask
pacman.draw.warpRightLoop:
	ds	0
	bm	pacman.draw.warpRightLoopEnd			; if we've reached the end of the bits, end the loop
	sl	1
	br	pacman.draw.warpRightLoop
pacman.draw.warpRightLoopEnd:
	lr	5, A						; save the bitmask
pacman.draw.warpEnd:

pacman.draw.sprite:						; draw the sprite
	SETISAR	PACMAN_X_REG
	; load x coordinate
	lr	A, I
	lr	2, A
	; load y coordinate
	lr	A, S
	lr	3, A
	; draw the sprite
	pi	sprite.draw

pacman.draw.end:
	pi	kstack.pop
	pk							; return from the subroutine

;---------------------------------------------------------------------------
; Draw Pac-Man lives 
;---------------------------------------------------------------------------

pacman.drawLives:
	; save return address
	lr	K, P
	pi	kstack.push

	; erase the lives display
pacman.drawLives.erase:
 	; load sidebar color
	li	SIDEBAR_COLOR
	lr	1, A

	; start row loop
	li	56
	lr	3, A
pacman.drawLives.erase.row:
	li	97
	lr	2, A
pacman.drawLives.erase.column:
	pi	plot

pacman.drawLives.erase.checkLoop:
	; check column
	ds	2
	lr	A, 2
	ci	76						; see if we've reached the left edge
	bnz	pacman.drawLives.erase.column

	; check row
	ds	3
	lr	A, 3
	ci	51
	bnz	pacman.drawLives.erase.row			; branch if > 0

	; load the number of lives
	SETISAR	PACMAN_LIVES_REG
	lr	A, S
	; draw a maximum of 4 lives
	ci	5
	bp	pacman.drawLives.setLivesCap
	lis	5
pacman.drawLives.setLivesCap:
	; use r10 as a loop counter
	lr	10, A
	
	; load the x coordinate
	li	78
	lr	11, A

pacman.drawLives.loop:
	; decrease the life counter
	ds	10
	; return if we're through
	bz	pacman.drawLives.end
	
	; color
	li	PACMAN_COLOR
	lr	1, A
	; x position
	lr	A, 11
	lr	2, A
	; y position
	li	52
	lr	3, A
	; sprite index
	li	13
	lr	4, A
	; draw the life graphic
	pi	sprite.draw
	
	; increase the X coordinate
	lis	5
	as	11
	lr	11, A
	ci	94
	br	pacman.drawLives.loop	

pacman.drawLives.end:
	; return from the subroutine
	pi	kstack.pop
	pk

;===========================================================================
; Music Functions 
;===========================================================================

;---------------------------------------------------------------------------
; Play Song 
;---------------------------------------------------------------------------
; plays a song from a memory location stored in DC0
; uses r3, r4, r5, r7
; taken from Pro Football

playSong:                       				
	lis	3						; A = 3
	lr	4, A						; A -> r4, r4 = 3

.playSongDelay1:      
	inc							; increase A
	bnz	.playSongDelay1					; [Branch if not zero] back to .psdly1
	am							; Memory adressed by DC0 is added to A, flags set  - get duration data
	lr	7, A						; A -> r7
	bz	.playSongEnd					; Go to end if zero
	lm							; Load memory into A (adressed by DC0) - get frequency data
	lr	5, A						; A -> r5

	; check to see if we should delay
	inc
	bnc  .playSongLoop					; it didn't roll over, play a note instead

.playSongPause:
	li	$ff
	lr	6, A
.playSongPauseLoop:
	ds	6						; pause counter
	bnz	.playSongPauseLoop
	ds	7						; duration of the pause
	bnz	.playSongPause

	; play the next note
	br	playSong

.playSongLoop:
	li	$80						; A= $80 
	outs	5						; Send  A -> port 5
	lr	A, 5						; r5 -> A

.playSongDelay2:
	inc							; increase A
	bnz	.playSongDelay2					; [Branch if not zero] back to .psdly2
	outs	5						; A -> port 5
	lr	A, 5						; r5 -> A

.playSongDelay3:
	inc							; increase A
	bnz	.playSongDelay3					; [Branch if not zero] back to .psdly3
	ds	4						; decrease r4
	bnz	.playSongLoop					; [Branch if not zero] back to .psloop
	lis	3						; A = 3
	lr	4, A						; A -> r4
	ds	7						; decrease r7
	bnz	.playSongLoop						; [Branch if not zero] back to .psloop
	br	playSong					; start over - branch to beginning

.playSongEnd:
	pop							; return from the subroutine

;---------------------------------------------------------------------------
; Dot-eating Sound 
;---------------------------------------------------------------------------

sfx.dotEating.play:
	; output dot-eating sound
	li	$80
	outs	5
	lis	0
	outs	5						; reset port
	; return from the subroutine
	pop

;===========================================================================
; Program Data 
;===========================================================================

	; k stack functions
	include "kstack.inc"

	; graphic data
	include "graphics.inc"

	; data for the maze, the layout, and pellets
	include "maze.inc"

    IF switch_music
	; game music and sounds
	include "music.inc"
    ENDIF

;===========================================================================
; Signature 
;===========================================================================

	; signature
	org [$800 + [game_size * $400] - $10]

signature:
	.byte	"Blackbird 2006"